open Parsetree
open State

let new_env env = function
    | TB ->
	{env with b = true}
    | TEM ->
	{env with em = not env.em}
    | TI ->
	{env with i = true}
    | TPL ->
	{env with u = 0; s = false; em = false; i = false; b = false;
	 tt = false}
    | TS ->
	{env with s = true; em = false}
    | TTT ->
	{env with tt = true}
    | TU ->
	if env.u < 3 then {env with u = env.u + 1} else env
    | TSize size ->
	{env with size = size}
    | TColor c ->
	{env with color = c}
    | Notag ->
	env

let rec simp_tag env t s d =
    match t with
    | TB ->
	if env.b then Tag (Notag, "", d) else
	Tag (TB, s, simp_red (new_env env t) d)
    | TEM ->
	if env.s then Tag (Notag, "", d) else
	Tag (TEM, s, simp_red (new_env env t) d)
    |	TI ->
	if env.i then Tag (Notag, "", d) else
	Tag (TI, s, simp_red (new_env env t) d)
    |	TPL ->
	if env.u = 0 && not env.s && not env.em && 
	  not env.i && not env.b && not env.tt then
	  Tag (Notag, "", d)
	else
	  (Tag (TPL, s,
		  simp_red {env with u = 0; s = false; em = false; i =
			    false; b = false; tt = false} d))
    |	TS ->
	if env.s && not env.em then Tag (Notag, "", d) else
	Tag (TS, s, simp_red (new_env env t) d)
    | TTT ->
	if env.tt then Tag (Notag, "", d) else 
	  Tag (TTT, s, simp_red (new_env env t) d)
    |	TU ->
	if env.u = 3 then Tag (Notag, "", d) else
	Tag (TU, s, simp_red (new_env env t) d)
    | TSize size ->
	if env.size = size then Tag (Notag, "", d) else
	Tag (TSize size, s, simp_red (new_env env t) d)
    | TColor c ->
	  if env.color = c then Tag (Notag, "", d) else
	  Tag (TColor c, s, simp_red (new_env env t) d)
    | Notag ->
	Tag (Notag, s, simp_red env d)
and simp_red env = function
    Tag (a, s, b) :: t ->
      add (simp_tag env a s b) (simp_red env t)
  | Space c :: t -> Space c :: simp_red env t
  | Text s :: t -> Text s :: simp_red env t
  | Nil :: t -> simp_red env t
  | [] -> []

let rec simp_space env = function
    Space c1 :: Space c2 :: t when env.tt = false ->
      simp_space env (Space c1 :: t)
  | Tag (a, b, c) :: t ->
      Tag (a, b, simp_space (new_env env a) c) :: simp_space env t
  | Space c1 :: (Tag (Notag, _, Space c2 :: t1) as t) :: t2 
    when env.tt = false ->
      simp_space env (t :: t2)
  | Space c :: t ->
      Space c :: simp_space env t
  | Text s :: t ->
      Text s :: simp_space env t
  | Nil :: t -> simp_space env t
  | [] -> []
      


let simp_spc tree =
  simp_space def_state tree

let simp_redun tree =
  simp_red def_state tree

let simp tree =
  simp_spc (simp_redun tree)
